@@ -20,11 +20,12 @@ import sys
 """
 # ----- port here. 5000 is flask; is moved to 8000 by gunicorn; to 80 by nginx
 CONFIG={  'filename':'~/.config/test/cfg.json',
-          #'recommended':"",
           'average':0,
           'blur':0,
           "container":"mkv",
+          "cmatrixkill":False,
           'datapath': os.path.expanduser("~/DATA/"),
+          'defaultbg':'ubu_2204.jpg',
           'expo':'auto',  # EXPO
           'framekind':"direct", # direct , histo , detect
           'FOURCC':'avc1',  # OUTFILE SAVE; XVID=notmkv DIVX=mkv or IYUV
@@ -36,10 +37,19 @@ CONFIG={  'filename':'~/.config/test/cfg.json',
           'laps':0,
           'Maxresolution':"1280x720", # in twocam mode, 1920x doesnt work
           'mmaga':'def',  # GAMMA
+          'mminput1':"~/.config/flashcam/inputdata1_",
+          'mminput2':"~/.config/flashcam/inputdata2_",
+          'mminput3':"~/.config/flashcam/inputdata3_",
+          'mminput4':"~/.config/flashcam/inputdata4_",
+          'mminput5':"~/.config/flashcam/inputdata5_",
+          'mminput1_cfg':"signal xxx;0;100;60;title1",
+          'mminput2_cfg':"box xxx;0;100;60;title2",
+          'mminput3_cfg':"dial xxx;0;100;60;title3",
+          'mminput4_cfg':"tacho xxx;0;100;60;title4",
+          'mminput5_cfg':"sub xxx;0;100;60;title5",
           'netport':8000, # GOOD
           'otate':0, # rotate 0
           'password':'aaa',
-          #'port':5000, # crap....
           'PIXELFORMAT':'YUYV',  # CAMERA SOURCE XVID=notmkv DIVX=mkv or IYUV
           'product':"",
           'qpassfile':"~/.pycamfw_userpass", # useless ??
@@ -59,6 +69,9 @@ CONFIG={  'filename':'~/.config/test/cfg.json',
 #CFG_DEBUG = True
 #CFG_DEBUG = False
 
+daq_servers = []
+generic_bg_image = None
+fullpath_bg_fixed_image = None
 
 #===========================================================
 #===========================================================
