@@ -41,7 +41,6 @@ import flashcam.config as config
 
 from  flashcam.stream_enhancer import Stream_Enhancer
 
-
 from flashcam import v4lc
 from flashcam.v4lc import set_gem, get_gem, tune_histo
 
@@ -53,12 +52,19 @@ import sys
 from notifator import telegram
 import threading
 
-# there is a problem with ttf fonts to cv2:
 from PIL import ImageFont, ImageDraw, Image
 
 from console import fg,bg
 import socket
 
+from flashcam.text_write import iprint,fonts_available,\
+    get_f_height, get_f_width, get_def_font,\
+    dial, signal_strength, text_box, tacho, \
+    get_color_based_on_brightness, \
+    MSG_FONT
+
+# also in stream_enhancer??!!
+SOCKET_GETHOSTNAME =socket.gethostname()
 
 try:
     import pyautogui # take screenshot
@@ -67,18 +73,23 @@ except:
 # -----------------------------------------------------------------
 
 
+from flashcam import mmapwr
+
 current_ip = None
 
 def is_int(n):
+    if str(n).find(".")>=0:  return False
+    if n is None:return False
     try:
-        float_n = float(n)
-        int_n = int(float_n)
+        float_n = float(n) # 0.0
+        int_n = int(float_n) #0.0
     except ValueError:
         return False
     else:
         return float_n == int_n
 
 def is_float(n):
+    if n is None:return False
     try:
         float_n = float(n)
     except ValueError:
@@ -101,6 +112,7 @@ def get_ip( myip ):
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         s.connect(("8.8.8.8", 80))
         current_ip = s.getsockname()[0]
+        ### current_ip = SOCKET_GETHOSTNAME
         s.close()
     return current_ip
 
@@ -117,33 +129,147 @@ def leftmost_txt_wid( txt ):
             minx = posx
     return minx
 
-def force_text( frame, overtext , posy = 400, bg_bgra = (255,0,255,0) , fg_bgra = (0,255,255,255) , fosize = 2*32):
+
+def force_center_text( frame, overtext , posy = 400, fg_bgr = (0,255,0) , fosize = 2*32):
     """
-    7segment LCD text in the middle
+    7segment LCD text in the middle is a good example
+    siglost, nocamera, overtext, baduser....
     """
-    posx = leftmost_txt_wid( overtext )# int(640/2 - len( str(overtext) )*32/2)
-    if posx<0: posx=10
-    position = ( posx ,posy)
-    fontpath = os.path.expanduser("~/.config/flashcam/digital-7.mono.ttf")
-    font = ImageFont.truetype(fontpath, fosize)
-
-    img_pil = Image.fromarray(frame).convert("RGBA")  # base img
-    txt_img = Image.new("RGBA", img_pil.size, bg_bgra)
-
-
-    #draw = ImageDraw.Draw(img_pil)
-    draw = ImageDraw.Draw(txt_img)
-    drtext =  f"{overtext}"# {overtextalpha}" # to be sure
-    draw.text( position,  drtext, font = font, fill = fg_bgra )
-
-    composite = Image.alpha_composite(img_pil, txt_img)
-    #composite.save(output_path)
-    #frame = np.array(img_pil)
-    frame = np.array(composite)
+    FONT = "utfh"
+    midl = 320
+    midl = midl - len(str(overtext))*get_f_width(FONT)//2 # center
+    frame = iprint( frame, drtext=str(overtext), font=FONT, position=(midl,posy), color_rgb=fg_bgr)
     return frame
 
 # -----------------------------------------------------------------
 
+
+class widget:
+    """
+    Every start, for 1 second the widgets will light up
+    """
+    def __init__(self,connector=1, typ = "signal", pos = (580,60), timeout=1, adaptive_position=True):
+        self.connector = int(connector)
+        if self.connector == 1: self.inputi = config.CONFIG['mminput1']
+        elif self.connector == 2: self.inputi = config.CONFIG['mminput2']
+        elif self.connector == 3: self.inputi = config.CONFIG['mminput3']
+        elif self.connector == 4: self.inputi = config.CONFIG['mminput4']
+        elif self.connector == 5: self.inputi = config.CONFIG['mminput5']
+        else:
+            print("X... nonexisting widget connector")
+            sys.exit(1)
+
+        self.title = "ooo"
+        self.inputi = os.path.expanduser( self.inputi )
+        self.typ = typ
+        self.position = pos
+        self.timeout = timeout # default timeout==1 sec
+        self.last = dt.datetime.now()
+        self.value0100 = 50
+        self.value = 12.3
+        self.raw = "test1 mA,test2 uSv"
+        #if typ != "sub":
+        self.adaptive_position = adaptive_position
+        #else:
+        #    self.adaptive_position = False
+        #
+        # I try to put extra DATA mmap file here
+        #
+        mmapwr.mmcreate( self.inputi  )
+        #print("******************  mm created",config.CONFIG["mminput1"])
+
+
+    def refresh(self):
+        """
+        read adn interpret values
+        value;min;max;
+        """
+        input1,input1ext = mmread_n_clear( self.inputi )
+        input1 = input1.strip()
+
+        if (input1 is not None) and (type(input1) == str) and (input1 in ['signal', 'dial', 'box', 'sub', 'tacho']):
+            self.last = dt.datetime.now() # refresh
+            print(">",input1, "Exteded==", input1ext)
+
+            self.typ = input1
+
+            res = input1ext.split(";")
+            self.raw = res[0].strip()  # table text box split ,comma,
+
+            print(res)
+            if is_int(res[0]):
+                self.value = float(res[0]) # BUG HERE???? 0.0 is int!?
+            elif is_float(res[0]):
+                self.value = float(res[0]) #
+            else:
+                self.value = res[0]
+            print("D... self.value == ", self.value, "  raw=", self.raw )
+
+            self.value0100 = 50
+            if (len(res)>2) and (type(self.value) == int ) or (type(self.value) == float):
+                mini,maxi=0,100
+                if is_float(res[1]): mini = float(res[1]) #
+                if is_float(res[2]): maxi = float(res[2]) #
+                if mini<maxi:
+                    self.value0100 = (self.value-mini)/(maxi-mini)*100
+                if self.value0100<0: self.value0100=0
+                if self.value0100>100: self.value0100=100
+
+            if len(res)>3: # timeout
+                timo = self.timeout
+                if is_float(res[3]): timo = float(res[3])
+                if timo>0.8 and timo<3600: # between 1 sec and 1 hour
+                    self.timeout = timo
+                    print("X... changing  timeout (1-3600sec)", timo)
+                else:
+                    print("X... uneligible timeout (1-3600sec)", timo)
+
+            if len(res)>4: # title
+                self.title = str(res[4])
+
+
+           #print(self.value)
+
+    def is_active(self):
+        delta = (dt.datetime.now()-self.last).total_seconds()
+        #print(self.connector, delta )
+        if delta<self.timeout:
+            return True
+        else:
+            return False
+
+
+    def display(self, frame ):
+        cposition = list(self.position)
+        radius = 30 # Guess - not systematic
+        if self.adaptive_position and self.typ != "sub":
+            cposition[0] = frame.shape[1] - radius * 2
+        if self.adaptive_position and self.typ == "sub":
+            cposition[1] = frame.shape[0] - radius
+
+        left_up_point = (cposition[0] - radius, cposition[1] - radius)
+        right_down_point = (cposition[0] + radius, cposition[1] + radius)
+        dcolor = get_color_based_on_brightness( frame, left_up_point , right_down_point)
+        #if self.is_active():
+        if self.typ == "signal":
+            frame = signal_strength( frame, position=cposition , percentage= self.value0100 , value=self.value, color=dcolor, title=self.title)
+        if self.typ == "dial":
+            frame = dial( frame, position=cposition , percentage= self.value0100 , value=self.value , color=dcolor, title=self.title)
+        if self.typ == "tacho":
+            frame = tacho( frame, position=cposition , percentage= self.value0100 , value=self.value , color=dcolor, title=self.title)
+        if self.typ == "box":
+            frame = text_box( frame, position=cposition , title = self.title  , values = str(self.value).replace(",","\n") , color=dcolor)
+        if self.typ == "sub":
+            #print("D... STARTING SUB PRINT")
+            frame = iprint( frame, self.value , MSG_FONT,
+                            position=cposition,color_rgb=dcolor)
+        return frame
+
+
+
+# -----------------------------------------------------------------
+
+
 class Camera(BaseCamera):
 
     # video_source = 0
@@ -162,6 +288,8 @@ class Camera(BaseCamera):
         #  - all is taken from BaseCam
         # res = "640x480"
         res = config.CONFIG["resolution"]
+
+
         print("i... init _ cam caleld with prod:",  config.CONFIG["product"] )
 
         # here, I need to  wait until the correct video is reported.... ????
@@ -326,16 +454,31 @@ class Camera(BaseCamera):
             nfrm = 0
             if frame is None:
                 height, width = 480, 640
-                blank_image = np.zeros((height,width,3), np.uint8)
-                # GRAY GREY 255 128 ---- half split image in a case ....happens too frequent..
-                blank_image[:,0:width//2] = (90,90,90)      # (B, G, R)
-                blank_image[:,width//2:width] = (150,150,150)
-                frame = blank_image
+                if config.generic_bg_image is None:
+                    config.fullpath_bg_fixed_image = f"{os.path.dirname(config.CONFIG['filename'])}/{config.CONFIG['defaultbg']}"
+                    print("i... BG IMAGE PATH", config.fullpath_bg_fixed_image)
+                    config.fullpath_bg_fixed_image = os.path.expanduser(config.fullpath_bg_fixed_image)
+                    print("i... BG IMAGE PATH", config.fullpath_bg_fixed_image)
+                    if config.fullpath_bg_fixed_image is not None:
+                        print("i... BG IMAGE READING ")
+                        config.generic_bg_image = cv2.imread( config.fullpath_bg_fixed_image)
+                    print("i... BG IMAGE READ: ", config.fullpath_bg_fixed_image)
+                #print("i... BG IMAGE GENE: ", config.generic_bg_image )
+                if config.generic_bg_image is not None:
+                    frame = config.generic_bg_image
+                else:
+                    frame = np.zeros((height,width,3), np.uint8)
+                frame = cv2.resize(frame, (width,height) )
+                # blank_image = np.zeros((height,width,3), np.uint8)
+                # # GRAY GREY 255 128 ---- half split image in a case ....happens too frequent..
+                # blank_image[:,0:width//2] = (10,10,10)      # (B, G, R)
+                # blank_image[:,width//2:width] = (30,30,30)
+                #frame = blank_image
 
             overtext = "XXXXXXXXXXX"
             overtext = get_ip( "" )
-            overtext = f"Device lost\non\n{overtext}"
-            frame = force_text( frame, overtext, posy = 200)
+            overtext = f"Device lost on {overtext}"
+            frame = force_center_text( frame, overtext, posy = 200, fg_bgr=(0, 0, 255) )
 
         return frame, cap
 
@@ -346,12 +489,13 @@ class Camera(BaseCamera):
         """
         vidnum None or -1 ...   check cap for filenames
         """
-        fullpath_fixed_image = "~/.config/flashcam/monoskop.jpg"
-        fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image)
+        config.fullpath_bg_fixed_image = "~/.config/flashcam/monoskop.jpg"
+        config.fullpath_bg_fixed_image = "~/.config/flashcam/ubu_2204.jpg"
+        config.fullpath_bg_fixed_image = os.path.expanduser( config.fullpath_bg_fixed_image)
         # debug print( cap, vidnum)
-        if not os.path.exists(fullpath_fixed_image):
-            print("X... monoskop doesnt exist")
-            fullpath_fixed_image = None
+        if not os.path.exists(config.fullpath_bg_fixed_image):
+            print("X... monoskop/ubuntu_2204.jpg doesnt exist")
+            config.fullpath_bg_fixed_image = None
 
         if not (vidnum is None) and (vidnum==-1):
             #
@@ -375,38 +519,24 @@ class Camera(BaseCamera):
                 return "image", frame, cap # repeat cap==image
 
             elif cap.find("clock")==0:
+                """
+                clock clocks clocks.jpg .....
+                """
                 #
                 # demanded 'living' jpg
                 #
                 height, width = 480, 640
                 blank_image = np.zeros((height,width,3), np.uint8)
-                blank_image[:,0:width//2] = (20,20,20)      # (B, G, R)
-                blank_image[:,width//2:width] = (25,25,25)
-                position = (10,150)
-
-                # FONTS https://www.1001fonts.com/search.html?search=digital
-                fontpath = os.path.expanduser("~/.config/flashcam/digital-7.mono.ttf")
-
-                font = ImageFont.truetype(fontpath, 4*32)
-                img_pil = Image.fromarray(blank_image)
-                draw = ImageDraw.Draw(img_pil)
+                blank_image[0:height//2,0:width//2] = (25,15,15)      # (B, G, R)
+                blank_image[0:height//2,width//2:width] = (15,25,15)
+                blank_image[height//2:height,width//2:width] = (15,15,25)
+                blank_image[height//2:height,0:width//2] = (25,15,25)
+                position = (10, 320)
                 drtext = dt.datetime.now().strftime("%H:%M:%S.%f")[:-5]
+                frame = iprint( blank_image, drtext,
+                                "di90" , position,  color_rgb=(0,255,0)  ) # green LED
 
-                #draw.text( position,  "国庆节/中秋节 快乐!", font = font, fill = (b, g, r, a))
-                b,g,r,a = 0,255,0,200
-                draw.text( position,  drtext, font = font, fill = (b, g, r, a))
-                frame = np.array(img_pil)
                 time.sleep(0.1) # from 85%cpu to 20% with 0.1;
-                # cv2.putText(
-                #     blank_image, #numpy array on which text is written
-                #     drtext, #text
-                #     position, #position at which writing has to start
-                #     #cv2.FONT_HERSHEY_SIMPLEX, #font family
-                #     font,
-                #     4, #font size
-                #     (249, 240, 250, 55), #font color
-                #     5) #font stroke
-                # frame = blank_image
                 return "image", frame, cap # repeat cap==image
 
             elif cap.find(".jpg")>=0:
@@ -416,26 +546,27 @@ class Camera(BaseCamera):
                 #print("i... static image mode")
                 capfull = os.path.expanduser( f"~/.config/flashcam/{cap}" )
                 if  os.path.exists(capfull):
-                    fullpath_fixed_image =  capfull
+                    config.fullpath_bg_fixed_image =  capfull
                     #print("W... image        exist", cap, " OK")
                 else:
-                    print("X... image doesnt exist", cap, "using monoskop")
+                    print("X... image doesnt exist", cap, "using monoskop/ubu_2204")
 
                 time.sleep(0.3) # from 85%cpu to 20% with 0.1
-                retimg = cv2.imread( fullpath_fixed_image)
-                retimg = cv2.resize(retimg, (640,480) )
+                if config.generic_bg_image is None:
+                    config.generic_bg_image = cv2.imread( config.fullpath_bg_fixed_image)
+                retimg = cv2.resize(config.generic_bg_image, (640,480) )
                 return "image", retimg, cap # repeat cap==image
         else:
             #==================================== monoskop added with IP ADDRESS in black
             # print("i...  camera mode .....   cap-or-image",type(cap), cap, vidnum)
             if cap is None and vidnum is None:
-                print("X... camera not accessible ... NO CAMERA FOUND ")
-                time.sleep(0.3)
-                frame = cv2.resize(cv2.imread( fullpath_fixed_image), (640,480) )
+                #print("X... camera not accessible ... NO CAMERA FOUND ")
+                time.sleep(0.2) # decrease framerate to 5fps
+                frame = cv2.resize(cv2.imread( config.fullpath_bg_fixed_image), (640,480) )
                 overtext = "XXXXXXXXXXX"
                 overtext = get_ip("")
-                overtext = f"No camera found on\n{overtext}"
-                frame = force_text(frame, overtext , posy = 25, fg_bgra = (100,100,250,255) ) # blue
+                overtext = f"No camera found on {overtext}"
+                frame = force_center_text(frame, overtext , posy = 35, fg_bgr=(0, 0, 255))
                 txt_img = "xxxxxxxxxxx"
 
                 return "image_forced",  frame, cap # repeat cap==image
@@ -464,7 +595,7 @@ class Camera(BaseCamera):
         global gamma_divide, gamma_multiply,gamma_setdef
         global gain_divide,gain_multiply,gain_setdef
         global expo_divide,expo_multiply,expo_setdef,  expovalue, gainvalue
-        global timelaps, rotate180
+        global timelaps, timelaps_triggered, rotate180
         global fixed_image # show not camera but image
         global zoom
         global resozoom
@@ -478,6 +609,17 @@ class Camera(BaseCamera):
         # print("i... staticmethod frames @ real -  enterred; target_frame==", target_frame)
         # ----- I need to inform SENH about the resolution
 
+
+        #
+        #  DEFINE INPUT Processing
+        #
+        mywidgets = []
+        mywidgets.append( widget(1,pos=(585,50) , typ = "signal") )
+        mywidgets.append( widget(2,pos=(585,150), typ = "dial") )
+        mywidgets.append( widget(3,pos=(585,250), typ= "tacho" ) )
+        mywidgets.append( widget(4,pos=(585,350), typ = "box" ) )
+        mywidgets.append( widget(5,pos=(220,450), typ = "sub" ) )
+
         senh = Stream_Enhancer( resolution = config.CONFIG["resolution"] ) # i take care inside
 
         senh.zmqtarget = None # initially
@@ -508,6 +650,7 @@ class Camera(BaseCamera):
         threshold    = int(config.CONFIG['threshold'])
         blur         = int(config.CONFIG['blur'])
         timelaps     = config.CONFIG['laps']
+        timelaps_triggered = False
         histogram    = config.CONFIG['Histogram']
         res          = config.CONFIG['resolution']
         speedx       = float(config.CONFIG['x'])
@@ -671,9 +814,11 @@ class Camera(BaseCamera):
                 ret = False
             elif ccoi1 == "image_forced":
                 # CAINI 3
-                cap, vidnum, s9009 = camera.init_cam()
-
-                print("D... CAINI 3 '''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''")
+                if random.random() < 0.1:
+                    cap, vidnum, s9009 = camera.init_cam()
+                    print("D... init cam tried'''''''''''''''''''''''''''", end="\r")
+                else:
+                    print("D... init cam skipped'''''''''''''''''''''''''", end="\r")
 
                 # *********  video works,  get capacities and go with EXPO GAIN
                 if not( (vidnum is None) or (vidnum == -1) ):
@@ -687,15 +832,6 @@ class Camera(BaseCamera):
             #print("D... ret==", ret)
             if ret: #********************************************************* operations
 
-                # FIXED IMAGE NOT ALLOWED DURING NORMAL CAMERA MODE - ONLY
-                # FIXED IMAGE NOT ALLOWED DURING NORMAL CAMERA MODE - ONLY  from 202304027
-                # FIXED IMAGE NOT ALLOWED DURING NORMAL CAMERA MODE - ONLY
-                #
-                # if fixed_image is not None:
-                #     fullpath_fixed_image = "~/.config/flashcam/"+fixed_image
-                #     fullpath_fixed_image = os.path.expanduser( fullpath_fixed_image )
-                #     if os.path.exists(fullpath_fixed_image):
-                #         frame = cv2.imread( fullpath_fixed_image)
 
                 #=====================================================
                 #  FIRST OPERATION ON FRAME ...  xzoom  switchresolution
@@ -819,7 +955,7 @@ class Camera(BaseCamera):
                         if expression=="overtext":
                             overtextalpha = 0
                         if expression=="baduser":
-                            print("X... realcam received info: baduser {value}")
+                            print(f"X... realcam received info: baduser=={value}")
                             baduser = str(value)
                         if (expression=="fixed_image"):
                             if (value is not None) and (value != "None"):
@@ -844,20 +980,26 @@ class Camera(BaseCamera):
                         if expression=="pausedMOTION":
                             if value is True:
                                 print("===================== DM is going STANDBY  ==================************==")
-                                CMD = "killall cmatrix &"
-                                try:
-                                    print("D... killing cmatrix .... there might be no process like that ...")
-                                    sp.Popen( CMD, shell=True)
-                                except:
-                                    print("X.... cannot kill cmatrix &")
+                                if config.CONFIG["cmatrixkill"]:
+                                    CMD = "killall cmatrix &"
+                                    try:
+                                        print("D... killing cmatrix .... there might be no process like that ...")
+                                        sp.Popen( CMD, shell=True)
+                                    except:
+                                        print("X.... cannot kill cmatrix &")
+                                else:
+                                    print("i... cmatrix is not run nor killed - see the config cfg.json")
                             if value is False:
                                 print("--------------------- DM is going to RUN RUN RUN ----------**************--")
-                                CMD = "xterm -fullscreen -e /usr/bin/cmatrix &"
-                                #CMD = "xterm  -e /usr/bin/cmatrix &"
-                                try:
-                                    sp.Popen( CMD, shell=True)
-                                except:
-                                    print("X.... cannot xterm  cmatrix")
+                                if config.CONFIG["cmatrixkill"]:
+                                    CMD = "xterm -fullscreen -e /usr/bin/cmatrix &"
+                                    #CMD = "xterm  -e /usr/bin/cmatrix &"
+                                    try:
+                                        sp.Popen( CMD, shell=True)
+                                    except:
+                                        print("X.... cannot xterm  cmatrix")
+                                else:
+                                    print("i... cmatrix is not run nor killed - see the config cfg.json")
 
 
 
@@ -869,26 +1011,21 @@ class Camera(BaseCamera):
                             print("i... TIMELAPS expression******************************* value=",value)
                             #senh.telegram_send_image(blocking= False) # it has an internal block (300sec)
 
-
-
-
-                        # ALREADY EXISTS from webform
-                        #if expression == "timelaps":
-                        #    print("i... timelaps changes .... ", timelaps. type(timelaps) )
-                        #    timelaps = value
-
-
-                        #
-                        # PROBLEM TO SOLVE
-                        #    i wait for re-demand from client........ seem to work.....
                         #
                         if expression == "switch_res":
                             #xzoom
-                            if type(value)==bool:
+                            if not config.CONFIG["ZoomResolution"]: # kick it immediatelly when not allowd in config
+                                print(f"X... {fg.red} === zoomresolution not allowed in config {fg.default}")
+                            elif type(value)==bool:
                                 switch_res = value #not(switch_res)
                                 print("i... ok ...ON/OFF")
 
-                                print("i...              EXPERIMENTAL SWITCH RESOLUTION now=", switch_res)
+                                print(f"i...       {fg.orange}   EXPERIMENTAL SWITCH RESOLUTION now=", switch_res,fg.default)
+                                print("i...   if it doesnt work check in conf.json config:              ")
+                                print("i...           'ZoomResolution':True                             ")
+                                print("i...           'Maxresolution':'1280x720' or '1920x1080          ")
+                                print("i...            'PIXELFORMAT':'MJPG' and not 'YUYV'              ")
+                                print("i...                                                             ")
                                 if switch_res:
                                     #maxres = "640x480"
                                     maxres = get_resolutions( vidnum )[-1] # from usb_check
@@ -995,8 +1132,6 @@ class Camera(BaseCamera):
                     # average  THIS IS HERE to be changed TOO (ACCUM)
                     # print("i.... average", average)
 
-                    # timelaps  THIS IS HERE to be changed TOO
-                    # print("i.... timelaps", timelaps)
 
                     #print("i... GAMMAS ", gamma, gammadef )
 
@@ -1147,7 +1282,10 @@ class Camera(BaseCamera):
                         senh.setbox(f"b  {blur}",  senh.blr)
                     if threshold>0:
                         senh.setbox(f"t  {threshold}",  senh.trh)
+
+                    # two ways of timelaps
                     if timelaps>0:
+                        timelaps_triggered = True
                         mycodec=config.CONFIG['FOURCC']
                         if mycodec == "DIVX":  #
                             senh.setbox(f"ld {timelaps}",  senh.lap)
@@ -1158,7 +1296,13 @@ class Camera(BaseCamera):
                         else:
                             senh.setbox(f"l  {timelaps} {mycodec}",  senh.lap)
                     if timelaps<0:
+                        timelaps_triggered = True
                         senh.setbox(f"l AS",  senh.lap)
+                    if timelaps == 0 and timelaps_triggered:
+                        print("D... timelaps was switched off now")
+                        senh.save_avi_close()
+                        timelaps_triggered = False
+
 
                     if histogram:
                         senh.setbox(f"h {hmean}",  senh.hist)
@@ -1228,7 +1372,7 @@ class Camera(BaseCamera):
                         senh.setbox(MODE_DM, senh.MODE, grayed = pausedMOTION) #---push UP to avoid DetMot
                         #print("D... detecting motion")
                         senh.detmo( average, blur)
-                        senh.chk_threshold( threshold )
+                        senh.chk_threshold( threshold , framekind=framekind)
                         #
                         # I need a way to block DETMO ....
                         # ??? BLUETOOTH ------- see later
@@ -1236,7 +1380,7 @@ class Camera(BaseCamera):
                         if senh.motion_detected: # saving avi on mation detect
                             # print("D... sav mot", senh.motion_detected)
                             if not pausedMOTION:
-                                senh.save_avi( seconds = -1, name = "_dm" , mycodec = config.CONFIG['FOURCC'], container =  config.CONFIG['container'])
+                                senh.save_avi( seconds = -1, name = "_dm" , mycodec = config.CONFIG['FOURCC'], container =  config.CONFIG['container'] , basecamera_string=f"{BaseCamera.nframes:07d} / {BaseCamera.capture_time} M" )
 
                     else:
                         senh.setaccum( average  )
@@ -1251,13 +1395,14 @@ class Camera(BaseCamera):
                     if timelaps>0:
                         mycodec=config.CONFIG['FOURCC']
                         senh.save_avi( seconds = timelaps,
-                                       basecamera_string=f"{BaseCamera.nframes:07d} / {BaseCamera.capture_time}",
+                                       basecamera_string=f"{BaseCamera.nframes:07d} / {BaseCamera.capture_time} L",
                                        mycodec = mycodec, frnum= BaseCamera.nframes)
 
                     if timelaps<0: # NEW...Save Every frame .. timelaps value is different from seconds value
                         mycodec=config.CONFIG['FOURCC']
-                        senh.save_avi( seconds = 0.001,
-                                       basecamera_string=f"{BaseCamera.nframes:07d} / {BaseCamera.capture_time}",
+                        # every frame AND take care of accumulation
+                        senh.save_avi( seconds = 0.001, # ASTRO
+                                       basecamera_string=f"{BaseCamera.nframes:07d} / {BaseCamera.capture_time} A",
                                        mycodec = mycodec, frnum= BaseCamera.nframes)
 
 
@@ -1273,17 +1418,55 @@ class Camera(BaseCamera):
 
                     # --- here I can touch frame:
                     if overtext is not None:
-                        frame = force_text( frame, overtext, posy = 400, fg_bgra= (0,255, 0, 255-int(overtextalpha)) )
+                        frame = force_center_text( frame, overtext, posy = 400, fg_bgr= (0,255, 0) ) #
+                        # DIAL --------
+                        post = (590,430)
+                        frame = dial(frame,post ,
+                                     color=(0,255,0) ,
+                                     radius = 30,
+                                     thickness = 15,
+                                     percentage= 100*overtextalpha/255 )
+                        # FO="v14"
+                        # frame = iprint( frame,
+                        #                 str(round(100*overtextalpha/255)),
+                        #                 FO,
+                        #                 position = (post[0]-2*get_f_width(FO)//2,
+                        #                             post[1] +get_f_height(FO)//2 ),
+                        #                 color_bgra= (0,255, 0, 255-int(overtextalpha))
+                        #                )
+
+                        frame = signal_strength( frame, position=(580,60) , percentage= 100*overtextalpha/255  )
+                        frame = text_box( frame, position=(320,240) )
+
+
                         if overtextalpha<255:
-                            overtextalpha+=0.2
+                            overtextalpha+=1.2
                         else:
                             overtext = None
 
-
                     if baduser is not None:
-                        frame = force_text( frame, f"P!=> {baduser}", posy = 440, fg_bgra= (0,0,255, 255), fosize = 32 )
+                        frame = force_center_text( frame, f"error! => {baduser}", posy = 440, fg_bgr= (0,0,255 ), fosize = 32 )
 
 
+                    #
+                    #  testing inputs
+                    #
+                    for i in mywidgets:
+                        i.refresh()
+                        #print("C...   widget #", i, end=" " )
+                        if i.is_active():
+                            frame = i.display(frame)
+                        #print()
+
+                    # input1,input1ext = mmread_n_clear( config.CONFIG['mminput1'] )
+                    # if input1 != "xxxxxx":
+                    #     print(">",input1, "Exteded==", input1ext)
+                    #     res = input1ext.split(";")
+                    #     ff = float(res[0])
+                    #     print(ff)
+                    #     frame = signal_strength( frame, position=(580,60) , percentage= ff  )
+                    # #else:
+                    # #    print("...")
 
                     if save_image_decor:  # camera_screenshot with all decor
                         print("D... HERE I SAVE  image camera_screenshot_decor")
