@@ -17,7 +17,6 @@ import os
 
 
 # run this program on each RPi to send a labelled image stream
-import socket
 import time
 from imutils.video import VideoStream
 import imagezmq
@@ -33,6 +32,31 @@ from console import fg
 from PIL import ImageFont, ImageDraw, Image
 
 from console import fg,bg
+from flashcam.text_write import iprint, fonts_available, get_f_height, get_f_width, get_def_font, set_def_font
+
+import flashcam.config as config # finally I need netport
+
+import skvideo.io
+
+
+# host name for once
+SOCKET_GETHOSTNAME =socket.gethostname()
+
+# =========================== DEFAULT FONT =====================
+# get_f_height("vt4")
+# #get_f_height("ty")
+# #get_f_height("fj6")
+# #get_f_height("re")
+# #get_f_height("v14")
+# #get_f_height("co")
+# #get_f_height("pr4")
+# get_f_height("nib")
+# #get_f_height("li")
+# #get_f_height("cu")
+# get_f_height("di")
+# #get_f_height("s8")
+
+set_def_font("di")
 
 class Stream_Enhancer:
     """
@@ -80,7 +104,7 @@ class Stream_Enhancer:
 
     motion_detected = False
     motion_detected_start = dt.datetime.now()
-    expire = 15 # expire motiondetected (earlier 2 sec)
+    expire = 10 # expire motiondetected (earlier 2 sec)
     aviopened = False
 
     avi_last = dt.datetime.now() - dt.timedelta(seconds=90000)
@@ -112,6 +136,7 @@ class Stream_Enhancer:
     telegramtrigger = False ## either BOOL or DATETIME
 
     aviout = None # I have a problem to seee saving the images to DaTA
+    skwriter_laps = None
 
     # this is used with setbox( "text", senh.TIME  ) _________
     def RC_COLOR(self,row,col):
@@ -276,73 +301,86 @@ class Stream_Enhancer:
         cv2.imshow("A", self.frame)
         cv2.waitKey(100)
 
-    def get_font_params(self, txt):
+    def get_font_params(self, txt, fname=None):
+        """
+        I think this is obsolete
+        """
         if not self.frame_ok: return
         self.font       = cv2.FONT_HERSHEY_SIMPLEX
         self.font       = cv2.FONT_HERSHEY_SIMPLEX
         self.font       = cv2.FONT_HERSHEY_DUPLEX
 
-        #fontpath = os.path.expanduser("~/.config/flashcam/small_pixel.ttf")
-        #self.font = ImageFont.truetype(fontpath, 16)
         self.font = cv2.FONT_HERSHEY_COMPLEX_SMALL
         self.font = cv2.FONT_HERSHEY_PLAIN  # needs offset
-        #ft = cv2.freetype.createFreeType2()
-        #ft.loadFontData(fontFileName='Ubuntu-R.ttf',  id=0)
 
         self.fontScale  = 0.85 * (self.frame.shape[1]/640) # 0.5 * (self.frame.shape[1]/640)
         self.lineType   = 1
-        self.text_width, self.text_height = cv2.getTextSize(txt,
-                                                  self.font,
-                                                  self.fontScale,
-                                                  self.lineType)[0]
+        self.text_width, self.text_height = cv2.getTextSize(txt,self.font,self.fontScale,self.lineType)[0]
+
+        if fname is None:
+            self.text_width, self.text_height = len(txt)*get_f_width(), get_f_height()
+        #self.text_width, self.text_height = 15, 15
 
 
 
 #------------------------------------------------------PUT TEXT --------------
     def textbox(self,txt, pos=[5,5], bgcolor=(0,0,0), fgcolor=(255,255,255) , target = None):
+
         if not self.frame_ok: return
 
-
-        if target is None:
-            framekind = self.frame
-        else:
-            framekind = target
-
-
-        self.get_font_params(txt)
-        #pos[1]+=self.text_height #
-
-        # letwidth = 25*4*fontScale
-        #        txt=txt+" w"+str(self.text_width)+" h"+str(self.text_height)+" @ "+str(pos[0])+","+str(pos[1])
+        self.get_font_params(txt )
         # top left 0 0
-        beginCorner=( pos[0]-1                , pos[1]- self.text_height-1 )
-        endCorner  =( pos[0]+self.text_width+1, pos[1]+3   )
+        beginCorner=( pos[0]-2                , pos[1]- self.text_height-1 )
+        endCorner  =( pos[0]+self.text_width+1, pos[1]+1   )
 
-        cv2.rectangle(framekind,
-                      beginCorner, endCorner,
-                      bgcolor, cv2.FILLED)
-        cv2.putText(framekind, txt,
-                    tuple(pos),
-                    self.font,
-                    self.fontScale,
-                    fgcolor,
-                    self.lineType)
+        if target is None or target == "direct":
+            cv2.rectangle(self.frame, beginCorner, endCorner, bgcolor, cv2.FILLED)
+            self.frame = iprint( self.frame,
+                             txt,
+                             get_def_font() ,
+                             tuple(pos),
+                             color_rgb= fgcolor#(255,255,255,0)
+                            )
+        elif target=="detect" and self.detect_frame is not None:
+            cv2.rectangle(self.detect_frame, beginCorner, endCorner, bgcolor, cv2.FILLED)
+            self.detect_frame = iprint( self.detect_frame,
+                             txt,
+                             get_def_font() ,
+                             tuple(pos),
+                             color_rgb= fgcolor#(255,255,255,0)
+                            )
+        elif target=="delta" and self.delta_frame is not None:
+            cv2.rectangle(self.delta_frame, beginCorner, endCorner, bgcolor, cv2.FILLED)
+            self.delta_frame = iprint( self.delta_frame,
+                             txt,
+                             get_def_font() ,
+                             tuple(pos),
+                             color_rgb= fgcolor#(255,255,255,0)
+                            )
+        elif target=="histo" and self.histo_frame is not None:
+            cv2.rectangle(self.histo_frame, beginCorner, endCorner, bgcolor, cv2.FILLED)
+            self.histo_frame = iprint( self.histo_frame,
+                             txt,
+                             get_def_font() ,
+                             tuple(pos),
+                             color_rgb= fgcolor#(255,255,255,0)
+                            )
 
-#    def timemark(self):
-#        # reserve 320
-#        self.get_font_params(txt)
-#        self.textbox(,  [self,5] )
 
 #----------------------------------------------- DEFINE GRID HERE-------
     def setbox(self, txt, rowcol, side="" , target = None, grayed = False, kompr=None):
         """
         uses textbox, pots on positions....
         """
-
-        if target is None:
-            frametgt = self.frame
-        else:
-            frametgt = target
+        frametgt = "direct"
+        if target is None or target=="direct":
+            frametgt = "direct" #self.frame
+        elif target == "detect":
+            frametgt = "detect"
+        elif target == "delta":
+            frametgt = "delta"
+        elif target == "histo":
+            frametgt = "histo"
 
         row,col=list(rowcol)
         if not self.frame_ok: return
@@ -351,7 +389,7 @@ class Stream_Enhancer:
         pos=[ self.posx_offs, self.posy_offs ]
         self.get_font_params(" WEB 2.3, ")
         WIDTH = self.text_width
-        self.get_font_params(txt)
+        self.get_font_params(txt )
 
         if side=="left":  col=0
         if side=="right":  col=-1
@@ -406,8 +444,8 @@ class Stream_Enhancer:
             if row==0:
                 if (col==0):
                     NOW = dt.datetime.now()
-                    date = NOW.strftime(".. %H:%M:%S %Y/%m/%d")
-                    me = socket.gethostname()
+                    date = NOW.strftime("__%H:%M:%S %Y/%m/%d")
+                    me = SOCKET_GETHOSTNAME #socket.gethostname()
                     txt = f"{date} {me} {__version__}"
                     # print("X... timemark requested")
                     self.textbox(txt, pos )
@@ -422,14 +460,14 @@ class Stream_Enhancer:
                     # RED GREEN CIRCLE:
                     #cirdiam = int(self.fontScale*10) # int(self.fontScale*16)
                     # not really # cirdiam = self.text_height + int(row*stretchy*self.text_height)
-                    cirdiam = int((self.text_height+3)/2*1.2)
+                    cirdiam = 1.2*get_f_height()/2
+                    cirdiam = round(cirdiam)
                     if NOW.second % 2 == 0:
-
                         #  fontsize 0.5=> 10   or 0.3 for RPI 320x240
-                        cv2.circle( frametgt, (cirdiam,cirdiam), cirdiam, (0,0,255), -1 )
+                        cv2.circle( self.frame, (cirdiam,cirdiam), cirdiam, (0,0,255), -1 )
                         # pass
                     else:
-                        cv2.circle( frametgt, (cirdiam,cirdiam), cirdiam, (0,255,0), -1 )
+                        cv2.circle( self.frame, (cirdiam,cirdiam), cirdiam, (0,255,0), -1 )
 
                     return
                 if col<3:
@@ -444,6 +482,7 @@ class Stream_Enhancer:
         #print(pos)
         bgcolor =  self.RC_COLOR(row,col)
         if grayed: bgcolor = (120,120,120)
+        #print( "D... # bgcolor", bgcolor )
         self.textbox(txt, pos , bgcolor= bgcolor , target = frametgt)
 
     #-------------------------------------------- ENDOF LABELS----------
@@ -617,18 +656,23 @@ class Stream_Enhancer:
 
 
 
-    def chk_threshold(self, threshold=100):
+    def chk_threshold(self, threshold=100,  framekind = None):
         """
+        preceeded by call to DETMO()
+        .
         compare threshold and eventually set DETECTED True
+        due to iprint xxx i need to fill a proper frameframekind
         """
-        # NOT HERE; this is a timelock - only a new seq self.motion_detected = False #  Here the flag is risen
+        self.motion_detected = False # I reset every test
         motion = False
-        self.detect_frame = self.frame.copy()
+        self.detect_frame = self.frame.copy() # FOR   -f detect,delta,direct
         self.frame_number+= 1
 
         #print( self.delta_frame)
-        if not (self.delta_frame is None):
+        if self.delta_frame is not None:
             # ------------- THIS IS TUNABLE FROM COMMANDLINE/CONFIG
+            # it cuts the scene in B-W and determines the MOTION AMOUNT
+            #
             thresh = cv2.threshold(self.delta_frame, threshold, 255,
                                 cv2.THRESH_BINARY)[1]
             # print(thresh)
@@ -636,13 +680,14 @@ class Stream_Enhancer:
             noz = 0 # non zero?
             try:
                 noz = cv2.countNonZero(thresh)
+                #  I dont use nonzeroes anymore.....
                 ok = True
             except:
                 ok = False
             if not(ok):
                 return
 
-            # -----------------------------------
+            # ----------------------------------- dilate TRESH
             thresh = cv2.dilate(thresh, None, iterations=3)
             cnts = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL,
                                     cv2.CHAIN_APPROX_SIMPLE)
@@ -660,7 +705,10 @@ class Stream_Enhancer:
                 (x, y, w, h) = cv2.boundingRect(c)
                 if (y+h<self.text_height*3) or (y>self.delta_frame.shape[0]-self.text_height*3):
                     continue
-                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,255,0),1)
+
+                # NO YES  DISPLAY green rectangles....
+                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,255,0),1)  #BGR = greens
+
                 if largesta<dearea:
                     largesta = dearea
                     largest = [c] # cheap trick, one instead of all
@@ -672,7 +720,7 @@ class Stream_Enhancer:
             # ------------------------------  MOTION FALSE --- AND CALCULATE
             motion = False
             # -----one in the list  for now.... if larger than 0.7 ==> i dont care...
-            for c in largest: # THE LIST OF ONE ELEMENT - TRICK
+            for c in largest: # THE LIST OF ONE ELEMENT - TRICK!!
                 area = cv2.contourArea(c)
                 # print("Largest: {}/{}  ... {:.2f}%".format(area, frame_area, area/frame_area*100) )
                 # rectangl 1% IS FINE. I pUSH to 0.1%
@@ -684,19 +732,33 @@ class Stream_Enhancer:
                 # what is here means detection
                 motion = True
                 (x, y, w, h) = cv2.boundingRect(c)
-                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,0,255),2)  # BGR
+                cv2.rectangle(self.detect_frame, (x,y),(x+w,y+h),(0,0,255),1)  # BGR
                 area=w*h # of the largest box
 
                 #  DM conditions in orange :  area/frame % / noz/frame % / cnts
-                dtext = "{:.1f}%/{:.1f}%/{}".format(
-                    area/frame_area*100, noz*100/frame_area, len(cnts) )
+                #dtext = "{:4.1f}%/{:4.1f}%/{}".format(
+                #    area/frame_area*100, noz*100/frame_area, len(cnts) )
+                #
+                # area of he largest RECT
+                #
+                # BUT I DONT USE NO-ZEROES
+                #dtext = f"{area/frame_area*100:4.1f}%/{noz*100/frame_area:4.1f}%/{len(cnts):2d}"
+                dtext = f"{area/frame_area*100:4.1f}%/{len(cnts):2d}"
 
                 # --- all with detection is done here.... detect and delta frames must exist then
-                self.setbox( f"{dtext}", self.dm ,  target = self.detect_frame )
-                self.setbox( f"{dtext}", self.dm ,  target = self.delta_frame )
-                self.setbox( f"{dtext}", self.dm    )
-                if "self.histo_frame" in locals():
-                    self.setbox( f"{dtext}", self.dm ,  target = self.histo_frame )
+                #if framekind is None or framekind == "direct":
+
+                # always print on main
+                self.setbox( f"{dtext}", rowcol=self.dm    )
+                if framekind == "detect":
+                    dtext = f"LARGEST={area/frame_area*100:4.1f}%/ NONZERO={noz*100/frame_area:4.1f}%/CONTOURS={len(cnts):2d}"
+                    self.setbox( f"* {dtext}", self.dm , target = "detect" )
+                elif framekind == "delta":
+                    ## it is BW - it may create a problem in iprint:
+                    self.setbox( f"{dtext}", self.dm ,  target = "delta" )
+                elif framekind == "histo":
+                    if "self.histo_frame" in locals():
+                        self.setbox( f"{dtext}", self.dm ,  target = "histo" )
 
 
             # all calculation done:
@@ -706,29 +768,27 @@ class Stream_Enhancer:
             if motion:
                 self.motion_detected_start = dt.datetime.now() # Set New Time
                 #print("i... NEW motion detected", self.motion_detected_start, self.frame_number)
-
             if motion:
                 self.motion_detected = True
                 #print("i... Motion ON")  # comming here mns already TRUE
 
-            # this was the protection not to save too long....
-            #if ((dt.datetime.now() - self.motion_detected_start).total_seconds() > self.expire) :
-            #    self.motion_detected = False # this kills
-            #    # print("i... Motion OFF expired", self.expire, self.frame_number)
-
 
             # this IS now the forced recording .....
             if ((dt.datetime.now() - self.motion_detected_start).total_seconds() < self.expire) :
-                #print(f"i...     {(dt.datetime.now() - self.motion_detected_start).total_seconds():.1f} s / {self.expire} s     " )
-                proc = (dt.datetime.now() - self.motion_detected_start).total_seconds() / self.expire
-                bar = 13 - int(proc*13)
-                if not motion: self.setbox( " "*bar, self.dm    )
+                if self.motion_detected == False:
+                    #print(f"i...   {(dt.datetime.now()-self.motion_detected_start).total_seconds():.1f} s/{self.expire}" )
+                    proc = (dt.datetime.now() - self.motion_detected_start).total_seconds() / self.expire
+                    bar = 13 - int(proc*13)
+                    #if not motion:
+                    self.setbox( " "*bar, self.dm    )
                 self.motion_detected = True
             else:
                 # it expired
-                if not motion:
-                    # and no motion is here
-                    self.motion_detected = False
+                pass
+                #if motion:
+                #    # and no motion is here
+                #    self.motion_detected = False
+
                     #print("i... Motion OFF", self.frame_number)
             #if len(largest)==0: # ----------- if no contour - set OFF
             #    self.motion_detected = False # this kills
@@ -737,7 +797,7 @@ class Stream_Enhancer:
 
 
 
-    def detmo(self,  dmaccumulate=0, dmblur = 0, threshold = 100 ):
+    def detmo(self,  dmaccumulate=0, dmblur = 0):
         """
         if there is an accumulation - process it
         if there is NO detection, rewrite the buffer
@@ -775,7 +835,7 @@ class Stream_Enhancer:
                 #frame = cv2.absdiff(grayA, grayB)
                 self.delta_frame = cv2.absdiff(grayB, cv2.convertScaleAbs(grayA))
             else:
-                print("i... None frame2")
+                print("i... No delta in detection, using frame")
                 self.delta_frame = self.frame
 
 
@@ -999,8 +1059,33 @@ class Stream_Enhancer:
 
 
 
-# --------------------------------------------------------save avi ------------
-# ------------------------------------------------- also ZMQ
+
+    #--- what happens if not 640x480 and xvid?  XVID....too lossy compression
+    #fourcc = cv2.VideoWriter_fourcc(*'XVID')
+    # https://learn.microsoft.com/en-us/windows/win32/medfound/video-fourccs
+    #
+    # fourcc = cv2.VideoWriter_fourcc(*'XVID')  # NOT a QUALITY
+    # fourcc = cv2.VideoWriter_fourcc(*'IYUV')  # SHOULD BE HUGE....
+    #
+    # --------------------------------------------------------save avi ------------
+    # ------------------------------------------------- also ZMQ and TELEGRAM
+    def save_avi_close(self):
+        self.skwriter_laps.close()
+        self.skwriter_laps = None
+        print(f"i... {bg.red}LAPS skwriter  Closed{bg.default}")
+
+    def assure_DATA(self):
+        """
+        just do the same as in uniwrec, check+create data dir
+        """
+        dir2create = os.path.expanduser("~/DATA/")
+        if not os.path.isdir(os.path.expanduser(dir2create)):
+            print(
+                f"D... trying to create directory {dir2create} for saving"
+            )
+            # result = False
+            os.mkdir(os.path.expanduser(dir2create))
+
     def save_avi(self, seconds =  1, name = "" ,  basecamera_string= None ,
                  mycodec = "DIVX", frnum= -1,
                  container = "avi",
@@ -1012,6 +1097,8 @@ class Stream_Enhancer:
         -1 with name="dm" for detectmotion
         frnum .. if >0 and seconds <0.1 => omit may omit accumulated
          ...
+        https://gist.github.com/docPhil99/a612c355cd31e69a0d3a6d2f87bfde8b
+        https://trac.ffmpeg.org/wiki/Encode/H.264
         """
         #print("i... save_avi", seconds, name)
         # -------------- one shot
@@ -1019,27 +1106,27 @@ class Stream_Enhancer:
 
 
         if seconds == 0:
+            #
+            # NO SAVING
+            #
             return
         elif seconds == -1: # <0 [ -1 now] means saving avi - motion detect or in uni...*******
             #
-            #  NEGATIVE :  means saving all......
-            #     USED ALSO and mainly FOR DETECTION
             #
+            #  NEGATIVE :  means saving all ......
+            #     -  USED  FOR DETECTION  ...
             #
-            # trigger is old now
-            # if not(self.telegramtrigger)==bool:
-            #     if self.telegramtrigger<=dt.datetime.now()+dt.timedelta(seconds=1):
-            #         self.telegramtrigger = dt.datetime.now()+dt.timedelta(seconds=2)
-            #         print(f"i... telegram preset to {self.telegramtrigger.strftime('%H:%M:%S')}")
 
-            WH = (self.frame.shape[0],self.frame.shape[1]) # SHIIIIIT 2hours
             WH = (self.frame.shape[1],self.frame.shape[0])
 
-            if not self.aviopened or  (dt.datetime.now()-self.avi_started).total_seconds()>TIMEOUT_NEWFILE:
-                me = socket.gethostname()
+            avi_age = (dt.datetime.now()-self.avi_started).total_seconds()
+            if not self.aviopened or avi_age>TIMEOUT_NEWFILE:
+                me = SOCKET_GETHOSTNAME # socket.gethostname()
                 filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
-                filename = os.path.expanduser("~/DATA/"+f"{me}{name}_{filename}_{mycodec}.{container}")
-                for i in range(3): print(filename)
+                self.assure_DATA()
+                filename = os.path.expanduser("~/DATA/"+f"{me}{name}_{filename}_{str(config.CONFIG['netport'])}_{mycodec}.{container}")
+                for i in range(3):
+                    print(f"i... {bg.green}LAPS<0:{filename}{bg.default}")
                 if (dt.datetime.now()-self.avi_started).total_seconds()>TIMEOUT_NEWFILE:
                     print("i... RLEASING VIDEOFILE ..... EVERY sec.==", TIMEOUT_NEWFILE)
                     self.aviout.release()
@@ -1047,38 +1134,51 @@ class Stream_Enhancer:
                     print("i... RELEASING VIDEOFILE ..... EVERY sec.==", TIMEOUT_NEWFILE)
                     self.avi_started = dt.datetime.now()
                 #
+                #
                 fourcc = cv2.VideoWriter_fourcc(*mycodec)  #
                 self.aviout = cv2.VideoWriter( filename , fourcc , fps, WH )
+                # #
+                # self.skwriter = skvideo.io.FFmpegWriter( f"{filename}.mp4",  outputdict={
+                #     '-vcodec': 'libx264',  #use the h.264 codec
+                #     '-crf': '0',           #set the constant rate factor to 0, which is lossless
+                #     '-preset':'veryslow'   #the slower the better compression, in princple, try
+                #     #other options see https://trac.ffmpeg.org/wiki/Encode/H.264
+                # })
+                # #
+                self.save_avi_counter = 0 # count saved frames
                 self.aviopened = True
-                self.aviout.write(self.frame)
-                self.aviout.write(self.frame)
-                self.aviout.write(self.frame)
+                for i in range(3): # doesnt help much to create a file
+                    #print(".", flush=True, end="")
+                    self.aviout.write(self.frame)
+                    #self.skwriter.writeFrame(self.frame[:,:,::-1])  #write the frame as RGB not BGR
 
 
-                #--- what happens if not 640x480 and xvid?  XVID I ALWAYS USED.... sometimes too big compression
-                #fourcc = cv2.VideoWriter_fourcc(*'XVID')
+            # ---- repeat the (later) printout
+            if basecamera_string is not None:
+                overtext=basecamera_string
+            else:
+                overtext = ";"
+            overtext = f"D {self.save_avi_counter:07d} / {overtext}"
+            #overtext = ' '.join(overtext[i:i+1] for i in range(0, len(overtext), 1))
+            position = ( 460, self.frame.shape[0]-1 ) # 480 on x-axis
+            self.frame = iprint(self.frame, str(overtext), font="di",
+                            position=position,color_rgb=(0,255,0) )
 
-            #elif (self.aviopened) and ( (dt.datetime.now()-self.avi_started).total_seconds()>86400): # it is opened but too long, one day....
-                #  print(self.aviout)
-
-            #if self.aviout:
-            #    #print("X")
             self.aviout.write(self.frame)
-            # print("                                        ",WH, self.frame.shape, fps, end  = "\r")
-            # self.aviout.release()
-            # plan telegram send to future
-            # bool means it is not set....
+            #self.skwriter.writeFrame(self.frame[:,:,::-1])  #write the frame as RGB not BGR
+            self.save_avi_counter+=1 # count saved frames
+
+            #
+            # If telegram => check the tripwire (time block)
             #
             if self.jtelegram and type(self.telegramtrigger)==bool:
                 self.telegramtrigger = dt.datetime.now()+dt.timedelta(seconds=self.telegram_tripwire) # TRIPWIRE 2 seconds
                 print("i... telegramtrigger preset to +2 sec.",self.telegramtrigger.strftime("%H:%M:%S"))
             # MOMENT WHEN AVI IS SAVED===========> imagezmq, telegram...
 
-
-
-
-            #print(self.zmqsender,self.zmqtarget)
-            # zmqtarget - if not NONE not initialized
+            #
+            # If ZMQ is initialized
+            #
             if  (self.zmqsender is None) and not (self.zmqtarget is None):
                 print(self.zmqsender,self.zmqtarget," INITIALIZING",f'tcp://{self.zmqtarget}:5555')
 
@@ -1109,30 +1209,31 @@ class Stream_Enhancer:
 
 
 
-        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
-        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
-        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
         elif seconds>0:
-            # ..0.01 means EVERY FRAME ..seconds is in principle different from 'timelapse' value
+        # ASTRO too
+        # -------------- timelapse if seconds>0: ******************************* MKV FOR LOOP
+        #
+        # ..0.01 means EVERY FRAME .. seconds is in principle different from 'timelapse' value
             WH = (self.frame.shape[1],self.frame.shape[0])
 
-            if not self.aviopened_laps:
-                me = socket.gethostname()
-                filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
-                fseconds = int(seconds)
-                filename = os.path.expanduser("~/DATA/"+ f"{me}_laps{fseconds:04d}_{filename}.mkv" )
-                for i in range(1):
-                    print(f"{bg.red}{filename}{bg.default}")
+            me = SOCKET_GETHOSTNAME # socket.gethostname()
+            filename = dt.datetime.now().strftime("%Y%m%d_%H%M%S")
+            fseconds = int(seconds)
+            self.assure_DATA()
+            filename = os.path.expanduser("~/DATA/"+ f"{me}_laps{fseconds:04d}_{filename}_{str(config.CONFIG['netport'])}_{mycodec}.mkv" )
 
+            if not self.aviopened_laps:
+                #
+                # AVI OPENED made True now
+                #
+                for i in range(1):
+                    print(f"i...{bg.green}LAPS>0:{filename}{bg.default}")
                 self.save_avi_counter = 0 # count saved frames
 
-                # https://learn.microsoft.com/en-us/windows/win32/medfound/video-fourccs
-                #
-                # fourcc = cv2.VideoWriter_fourcc(*'XVID')  # NOT a QUALITY
-                # fourcc = cv2.VideoWriter_fourcc(*'IYUV')  # SHOULD BE HUGE....
                 fourcc = cv2.VideoWriter_fourcc(*mycodec)  #
-
                 self.aviout_laps = cv2.VideoWriter( filename , fourcc , fps, WH  )
+
+
                 self.aviopened_laps = True
                 # self.avi_last = dt.datetime.now()
 
@@ -1158,25 +1259,34 @@ class Stream_Enhancer:
                 # for i in range(300,300+ self.save_avi_counter):
                 #     self.frame[100,i]=[255,255,255]
                 if basecamera_string is not None:
-                    # main information to be printed==basecamerastring
-                    overtext=basecamera_string #dt.datetime.now().strftime("%H:%M:%S.%f")[:-4] #"1234567890 12:34:55.38   1 2 : 3 4 : 5 5 . 3 8   "
-                    overtext = f"{self.save_avi_counter:06d} / {overtext}"
-                    fontpath = os.path.expanduser("~/.config/flashcam/small_pixel.ttf")
-                    position = ( 480, self.frame.shape[0]-8 ) # 480 on x
-                    font = ImageFont.truetype(fontpath, 6)
-                    img_pil = Image.fromarray(self.frame)
-                    draw = ImageDraw.Draw(img_pil)
-                    draw.fontmode = "1" # NO ANTIALIASING
-                    overtext = ' '.join(overtext[i:i+1] for i in range(0, len(overtext), 1))
-                    drtext =  str(overtext) # to be sure
-                    b,g,r,a = 255,255,255,0
-                    draw.text( position,  drtext, font = font, fill = (b, g, r, a))
-                    self.frame = np.array(img_pil)
+                    overtext=basecamera_string
+                    overtext = f"L {self.save_avi_counter:07d} / {overtext}"
+                    #overtext = ' '.join(overtext[i:i+1] for i in range(0, len(overtext), 1))
+                    #drtext =  str(overtext) # to be sure
+                    position = ( 460, self.frame.shape[0]-9 ) # 480 on x-axis
+                    self.frame = iprint(self.frame, str(overtext), font="di",
+                                   position=position,color_rgb=(0,255,0) )
 
                 self.aviout_laps.write(self.frame)
+
+                if not self.skwriter_laps:#
+                    fnama = f"{filename}.mp4"
+                    print(f"i... SAVING with ffmpeg TO: {fnama}")
+                    self.skwriter_laps = skvideo.io.FFmpegWriter( fnama,  outputdict={
+                        '-vcodec': 'libx264',  #use the h.264 codec
+                        '-crf': '0',           #set the constant rate factor to 0, which is lossless
+                        '-tune':'stillimage', #
+    #                    '-movflags': '',
+    #                    '+faststart':'',
+                        '-preset':'veryslow'   #the slower the better compression, in princple, try
+                        #other options see https://trac.ffmpeg.org/wiki/Encode/H.264
+                    })
+                    #
+                self.skwriter_laps.writeFrame(self.frame)#[:,:,::-1])  #write the frame as RGB not BGR
+
                 self.avi_last = dt.datetime.now()
                 #@{dt.datetime.now().strftime('%H:%M:%S.%f')[:-4]}
-                print(f"i... saving timelaps  #{self.save_avi_counter:06d}   [ {basecamera_string} ]" )
+                print(f"i... saving timelaps  #{self.save_avi_counter:07d}   [ {basecamera_string} ]" )
 
 
 
