@@ -35,9 +35,9 @@ import math
 import requests  # crosson=CROSSON
 from PIL import ImageFont, ImageDraw, Image
 
-from uniwrec_keys import remap_keys
-from uniwrec_manip import disp_mutext, rotate_image, adjust_gamma, make_measurements, matrix_undistort
-from uniwrec_io import setupsave, get_stream, get_FILE_REDCROSS
+from flashcam.uniwrec_keys import remap_keys
+from flashcam.uniwrec_manip import disp_mutext, rotate_image, adjust_gamma, make_measurements, matrix_undistort
+from flashcam.uniwrec_io import setupsave, get_stream, get_FILE_REDCROSS
 
 # ------------------problem with SHIFT
 # from pynput.keyboard import Key, Listener
@@ -817,7 +817,7 @@ Recommendation:
                     frame = adjust_gamma(frame, local_gamma)
 
                 if savepngcont:
-                    sfilename = datetim<e.datetime.now().strftime("%Y%m%d_%H%M%S")
+                    sfilename = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
                     sname = "snapshot"
                     sfilename = f"{sme}_{sfilename}_{sname}.png"
                     sfilename = os.path.expanduser("~/DATA/" + sfilename)
@@ -968,9 +968,12 @@ T ... use tracker2
                 # UBUNTU 22 - ctrl and alt give 227 and 233 resp
                 #
                 if key != -1:
-                    print(f" *key== {key}   /{chr(0xFF&key)}/  .. {c} : {s}  >>>.      ")
+                    #print(f" *key== {key}   /{chr(0xFF&key)}/  .. {c} : {s}  >>>.      ")
                     key, CTRL = remap_keys(key, CTRL ) # make compatible with waitKey()
-                    print(f" ... --------------------------------------------- remapped {key}")
+                    try:
+                        print(f" ... --------------------------------------------- remapped {key} : {chr(key)}")
+                    except:
+                        print(f" ... ----------------------------------------------r-m-p-ed {key} : {key}")
                     # if SHIFT: key = ord(chr(key).upper())
 
 
@@ -1049,6 +1052,7 @@ T ... use tracker2
                     print("i... it was version:", __version__)
                     sys.exit(1)
 
+                # ------------------- X just x
                 if (frame is not None) and (rpi_name != "") and (key == ord("x") and not CTRL):
                     print("x PRESSED! - expand 1x 2x")
                     if expande == 2:
@@ -1058,15 +1062,15 @@ T ... use tracker2
                         print("x PRESSED! - expand  2x")
                         expande = 2
 
-                # switch RES s-x
-                if (frame is not None) and (rpi_name != "") and (key == ord("X") and (not CTRL) ):
+                # switch RES c-x  - consistent to  z,s-z .... x,c-x,s-x
+                if (frame is not None) and (rpi_name != "") and (key == ord("x") and (CTRL) ):
                     print("X PRESSED! - resolution to MAX ")
                     post_addr = videodev.replace("/video", "/cross")
                     post_data = {"switch_res_on": "SWITCH_RES_ON"}
                     post_response = requests.post(url=post_addr, data=post_data)
 
-                # switch RES  c-x
-                if (frame is not None) and (rpi_name != "") and (key == ord("x") and CTRL):
+                # switch RES  s-x
+                if (frame is not None) and (rpi_name != "") and (key == ord("X") and (not CTRL) ):
                     print("Ctrl-X PRESSED! - resolution to 640 ")
                     post_addr = videodev.replace("/video", "/cross")
                     post_data = {"switch_res_off": "SWITCH_RES_OFF"}
