@@ -290,6 +290,9 @@ def get_v_m_r( vid="/dev/video0" ):
 
 
 def is_int(n):
+    if str(n).find(".")>=0:  return False
+    if n is None:return False
+
     try:
         float_n = float(n)
         int_n = int(float_n)
