@@ -8,6 +8,9 @@ This is flask interface
 from flashcam.version import __version__
 from fire import Fire
 import sys
+
+import xml.etree.ElementTree as ET # decoding s
+
 #
 # ======================== FIRST THING!!!!!!!!!!!!!!!!!!
 #
@@ -15,6 +18,7 @@ from flashcam import config
 
 config.CONFIG['filename'] = "~/.config/flashcam/cfg.json"
 
+
 if not config.load_config( create_if_nex=True ):
     print("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx")
     print("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXx")
@@ -48,6 +52,8 @@ from console import fg,bg
 
 import glob
 
+
+
 class RegexConverter(BaseConverter):
     def __init__(self, url_map, *items):
         super(RegexConverter, self).__init__(url_map)
@@ -55,11 +61,6 @@ class RegexConverter(BaseConverter):
 
 
 
-
-
-
-
-
 from importlib import import_module
 import os
 import sys  #exit
@@ -69,7 +70,6 @@ from flask import jsonify
 
 import getpass
 
-
 import datetime as dt
 import time
 
@@ -97,6 +97,7 @@ from flashcam.mmapwr import mmwrite
 PANTILTHAT = False
 # instead I want it for xzoom positioning
 xzoom_is_set = False
+timelaps_ptz = None
 try:
     import pantilthat
 except:
@@ -120,6 +121,11 @@ except:
 
 
 
+# ------------------------------------------------------
+# ------------------------------------------------------
+# ------------------------------------------------------
+
+
 app = Flask(__name__)
 
 # for regex
@@ -572,6 +578,7 @@ def cross():
 
         if request.form.get('overtext') is not None:
             digits = request.form.get('overtext')
+            #print("D... writing:", digits)
             mmwrite(f"overtext {digits}" )
 
 
@@ -589,6 +596,7 @@ def cross():
         # ---------------------------------------------- Cross controls
         if cross_on:
             # cross movement
+            print("D... ARROWS (@web) cross is on" )
             if request.form.get('left2') == 'LEFT2':
                 print("left2")
                 cross_dx-= 50
@@ -653,8 +661,8 @@ def cross():
                 print("PAN <-")
                 if (pantilthat.get_pan()+8)<=90:
                     pantilthat.pan( pantilthat.get_pan()+8 )
-        elif xzoom_is_set:
-            print("D... (web) true xzoom_is_set=", xzoom_is_set)
+        elif xzoom_is_set: # I am in zoom-mode
+            print("D... ARROWS(@web)  xzoom_is_set=TRUE (artif.zoom.ON)" )
             if request.form.get('down') == 'DOWN':
                 print("xzoom v")
                 mmwrite(f"switch_res D" )
@@ -667,16 +675,16 @@ def cross():
             if request.form.get('left') == 'LEFT':
                 print("xzoom <-")
                 mmwrite(f"switch_res L" )
-        else:
-            print("D... (web) false xzoom_is_set=", xzoom_is_set)
+        else:# i am not in zoom mode
+            print("D... ARROWS(@web) xzoom_is_set=FALSE (no action)")
             if request.form.get('down') == 'DOWN':
-                print("NO v")
+                print("NO action v")
             if request.form.get('up') == 'UP':
-                print("NO ^")
+                print("NO action ^")
             if request.form.get('right') == 'RIGHT':
-                print("NO ->")
+                print("NO action ->")
             if request.form.get('left') == 'LEFT':
-                print("NO <-")
+                print("NO action <-")
 
 
 
@@ -810,7 +818,7 @@ def verify_password(username, password):
 
 
 @app.route('/about')
-# @auth.login_required
+@auth.login_required
 def about():
     global remote_ip
     """about page.  Returns html from .config on cam"""
@@ -884,7 +892,7 @@ def index30():
 #  all PNG with structure (/about)
 #
 @app.route('/<regex("[abcdef0-9]{4}_[abcdef0-9]{4}_[abcdef0-9]{4}"):uid>.png')
-# @auth.login_required
+@auth.login_required
 def hexa_png(uid):
     return None
     # global  save_bg, frame_bg, frame_bg2, save_fg, frame_fg, frame_fg2, frame_mask, frame_mask_inv #, BGFILE
@@ -917,13 +925,12 @@ def hexa_png(uid):
 
 
 
-
 #
 #  all JPG with structure (/about)
 #
 @app.route('/<regex("[abcdef0-9]{4}_[abcdef0-9]{4}_[abcdef0-9]{4}"):uid>.jpg')
 @app.route('/<regex("[abcdef0-9]{4}_[abcdef0-9]{4}_[abcdef0-9]{4}_[0-9_]+"):uid>.jpg')
-# @auth.login_required
+@auth.login_required
 def hexa_jpg(uid):
     global  save_bg, frame_bg, frame_bg2, save_fg, frame_fg, frame_fg2, frame_mask, frame_mask_inv #, BGFILE
     print("i...  REGEX PAGE JPG ",uid)
@@ -1093,15 +1100,163 @@ def background():
 
 
 
+
+######################################################################################
+######################################################################################
 #
-#   in tinycam  windows  webcamxp   jpg
+#      PTZ BUT FOR Swann  DVR16-1599    -----------
+#
+######################################################################################
+######################################################################################
+
+def extract_value(data, tag, make_int=False, invert_int=False):
+    try:
+        root = ET.fromstring(data)
+        value = root.find(f'.//{tag}')
+        vt = value.text
+        if make_int:
+            vt = int(vt)
+            if invert_int:
+                vt = -vt
+        if (vt == "0") or (vt == 0):
+            return None
+        return vt if value is not None else None
+    except ET.ParseError:
+        return None
+    except:
+        return None
+
+
+@app.route('/ISAPI/PTZCtrl/channels/<regex("[1-9]+"):channel>/continuous', methods = ['GET', 'POST', 'PUT'])
+@app.route('/ISAPI/System/Video/inputs/channels/<regex("[1-9]+"):channel>/focus', methods = ['GET', 'POST', 'PUT'])
+@app.route('/ISAPI/System/Video/inputs/channels/<regex("[1-9]+"):channel>/iris', methods = ['GET', 'POST', 'PUT'])
+@app.route('/ISAPI/Image/channels/<regex("[1-9]+"):channel>/ircutFilter', methods = ['GET', 'POST', 'PUT'])
+@app.route('/ISAPI/PTZCtrl/channels/<regex("[1-9]+"):channel>/presets/<regex("[1-9]+"):preset>/goto', methods = ['GET', 'POST', 'PUT'])
+#  <PTZData><zoom>0</zoom></PTZData>b'<FocusData><focus>-60</focus></FocusData>'
+# b'<IrisData><iris>60</iris></IrisData>'
+# b'<PTZData><pan>60</pan><tilt>60</tilt></PTZData>'
+@auth.login_required
+
+def api_echo(channel=None, preset=None):
+
+    global xzoom_is_set # for xzoom resolution and newpantilt
+    global timelaps_ptz # INDEX
+
+    if request.method == 'GET':
+        print( "ECHO: GET\n" )
+
+    elif request.method == 'POST':
+        print( "ECHO: POST\n")
+        for i in request.args.keys():
+            print(i)
+
+    elif request.method == 'PATCH':
+        print( "ECHO: PACTH\n")
+
+    elif request.method == 'PUT':
+        data = request.data.decode('utf-8')
+        focus = iris = zoom = pan = tilt = irfilter = None
+
+        preset1 = preset
+        if (preset1 is not None) and (type(preset1) is str):
+            preset1 = int(preset1)
+        if '<FocusData>' in data:
+            focus = extract_value(data, 'focus', make_int=True, invert_int=True)
+        if '<IrisData>' in data:
+            iris = extract_value(data, 'iris', make_int=True)
+        if '<IrcutFilter>' in data:
+            irfilter =  extract_value(data, 'IrcutFilterType' )
+            # irfilter = extract_value(data, 'irfilter')
+        if '<PTZData>' in data:
+            zoom = extract_value(data, 'zoom', make_int=True)
+            pan = extract_value(data, 'pan', make_int=True)
+            tilt = extract_value(data, 'tilt', make_int=True)
+
+        if not all(value is None for value in [focus, iris, zoom, pan, tilt, irfilter, preset1]):
+            print("_______________________________________________________")
+            print(f"  pan      tilt        zoom       iris       focus         irfilter     preset     ")
+            print(f"  {pan}     {tilt}       {zoom}      {iris}         {focus}           {irfilter}        {preset1}     ")
+            print(f"")
+
+            switchzoom = 0
+
+            if (config.CONFIG["ZoomResolution"]) and (zoom is not None) and (int(zoom) > 0):
+                mmwrite(f"switch_res True" )
+                print(f"i... {fg.orange} ZOOM IN {fg.default}")
+                xzoom_is_set = True
+            if (config.CONFIG["ZoomResolution"]) and (zoom is not None) and (int(zoom) < 0):
+                mmwrite(f"switch_res False" )
+                print(f"i... {fg.orange} ZOOM OUT {fg.default}")
+                xzoom_is_set = False
+
+            if (config.CONFIG["ZoomResolution"]) and ((pan is not None) or (tilt is not None)): # ONLY  while in xzoom
+                if xzoom_is_set: # ---------- I am in xzoom  mode..... ------
+                    print(f"i... {fg.orange} experimental remote ctrl.: arrows {fg.default}")
+                    if (pan is not None) and (pan < 0):
+                        print("xzoom <-")
+                        mmwrite(f"switch_res L" )
+                        time.sleep(0.4)
+                    if (pan is not None) and (pan > 0):
+                        print("xzoom ->")
+                        mmwrite(f"switch_res R" )
+                        time.sleep(0.4)
+                    if tilt is not None and tilt > 0:
+                        print("xzoom ^")
+                        mmwrite(f"switch_res U" )
+                        time.sleep(0.4)
+                    if tilt is not None and tilt < 0:
+                        print("xzoom v")
+                        mmwrite(f"switch_res D" )
+                        time.sleep(0.4)
+
+            if (iris is not None):
+                if iris > 0:
+                    print("iris ^ /expo")
+                    mmwrite(f"expo_multiply True" )
+                if iris < 0:
+                    print("iris v / expo")
+                    mmwrite(f"expo_divide True" )
+            if (focus is not None):
+                if focus > 0:
+                    print("focus ^ / gain")
+                    mmwrite(f"gain_multiply True" )
+                if focus < 0:
+                    print("focus v / gain")
+                    mmwrite(f"gain_divide True" )
+            if (irfilter is not None):
+                print("D.... TODO  day night")
+                if irfilter == "auto":
+                    print("reset iris and focus   / expo and gain")
+                    mmwrite(f"gain_setdef True" )
+                    time.sleep(0.4)
+                    mmwrite(f"expo_setdef True" )
+
+
+        # print( f"ECHO: PUT    * * * *      preset=={preset}  channel={channel}    ")
+        # #print(request.json)
+        # print(request.data)
+        # #print(f"{request.json=}")
+        # #print(f"{request.get_json()=}")
+        # print("-------------------- ")
+
+    elif request.method == 'DELETE':
+        print( "ECHO: DELETE" )
+    return "ok", 200
+
+
+#   ENTRY POINT OF:                #   /tinycam ANDROID    application/
+#   Camera Type:    windows -  webcamxp   jpg ............... ( Better is Swann / DVR16-1500 !!!)
 #
 @app.route('/ptz')
 def ptz_windowsxp():
     """
     PTZ - 2 web variants , one is with real ptz (Crap) 2nd is a trick
     """
-    timelapses = [0,1,15,60]
+    global xzoom_is_set # for xzoom resolution and newpantilt
+    global timelaps_ptz # INDEX
+
+    # real time in seconds:
+    timelapses = [15, 1, 0] #[0,1,15,60]
 
     mx = request.args.get('movex')
     my = request.args.get('movey')
@@ -1109,6 +1264,7 @@ def ptz_windowsxp():
     zoom = request.args.get('zoom')
     print(f"i... {fg.yellow}  DX={mx}   DY={my}  SRC={src}  ZOOM={zoom}  {fg.default}")
 
+    # -------------------------------------------- try PTZ HAT, but deprecated.......
     ptz_ok = False
     try:
         pantilthat.show() # get_tilt()
@@ -1137,23 +1293,44 @@ def ptz_windowsxp():
                 if (pantilthat.get_pan()-1)>=-90:
                     pantilthat.pan( pantilthat.get_pan()-1 )
     else:
+        # ----------------------------------------------------------- CURRENT INTERPRETATION:
         #
         # checking timelaps here.....
         # I dont remember here what is the STATE
         #
-        print(f"i... {fg.orange} experimental remote cont.: dx=-1= laps 0s, 15s {fg.default}")
-        timelaps = 0
-        try:
+        if xzoom_is_set: # ---------- I am in xzoom  mode..... ------
+            print(f"i... {fg.orange} experimental remote ctrl.: arrows {fg.default}")
             if mx is not None and int(mx) == -1:
-                timelaps = 0
-                mmwrite(f"timelaps {timelaps}" )
+                print("xzoom <-")
+                mmwrite(f"switch_res L" )
+            if mx is not None and int(mx) ==  1:
+                print("xzoom ->")
+                mmwrite(f"switch_res R" )
+            if my is not None and int(my) > 0:
+                print("xzoom ^")
+                mmwrite(f"switch_res U" )
+            if my is not None and int(my) < 0:
+                print("xzoom v")
+                mmwrite(f"switch_res D" )
+        else:
+            print(f"i... {fg.orange} experimental remote ctrl.: dx=-1= laps 0s, 15s {fg.default}")
+            #timelaps = 0
+            #try:# ..... not sure about states of mx my......
+            if mx is not None and int(mx) == -1:
+                timelaps_ptz = None
+                mmwrite(f"timelaps 0" ) # RESET LAPS
                 time.sleep(0.4)
-                mmwrite(f"expo_setdef True" )
+                mmwrite(f"expo_setdef True" ) # RESET EXP
 
             if mx is not None and int(mx) ==  1:
-                timelaps = 15
-                mmwrite(f"timelaps {timelaps}" )
-            #
+                if timelaps_ptz is None:
+                    timelaps_ptz = 0 # index
+                else:
+                    timelaps_ptz += 1
+                    if timelaps_ptz >= len(timelapses): timelaps_ptz = 0
+                settimelaps = timelapses[timelaps_ptz] # real time in seconds
+                mmwrite(f"timelaps {settimelaps}" )
+                #
             if my is not None and int(my) > 0:
                 print(f"i... {fg.orange} EXPO UP {fg.default}")
                 mmwrite(f"expo_multiply True" )
@@ -1161,23 +1338,29 @@ def ptz_windowsxp():
                 print(f"i... {fg.orange} EXPO DOWN {fg.default}")
                 mmwrite(f"expo_divide True" )
 
-        except:
-            pass
-        # if mx is not None or my is not None: mmwrite(f"timelaps {timelaps}" )
+            #except:
+            #    pass
+            # if mx is not None or my is not None: mmwrite(f"timelaps {timelaps}" )
 
         switchzoom = 0
-        try:
-            if zoom is not None and int(zoom) > 0:
-                mmwrite(f"switch_res True" )
-                print(f"i... {fg.orange} ZOOM IN {fg.default}")
-            if zoom is not None and int(zoom) < 0:
-                mmwrite(f"switch_res False" )
-                print(f"i... {fg.orange} ZOOM OUT {fg.default}")
-        except:
-            pass
+        if zoom is not None and int(zoom) > 0:
+            mmwrite(f"switch_res True" )
+            print(f"i... {fg.orange} ZOOM IN {fg.default}")
+            xzoom_is_set = True
+        if zoom is not None and int(zoom) < 0:
+            mmwrite(f"switch_res False" )
+            print(f"i... {fg.orange} ZOOM OUT {fg.default}")
+            xzoom_is_set = False
 
     return "ok", 200
 
+
+
+#==================== IN FACT == THIS IS THE START OF THE CAMERA =====  REAL ENTRY @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
+#==================== IN FACT == THIS IS THE START OF THE CAMERA =====
+#==================== IN FACT == THIS IS THE START OF THE CAMERA =====
+#==================== IN FACT == THIS IS THE START OF THE CAMERA =====
+#==================== IN FACT == THIS IS THE START OF THE CAMERA =====
 #==================== IN FACT == THIS IS THE START OF THE CAMERA =====
 # with asking Camera()    I create an "instance" (static!) of the class
 # and call init_cam
@@ -1191,6 +1374,9 @@ def ptz_windowsxp():
 @app.route('/cam_3.jpg')
 #@app.route('/cam_<int(cam_id):[0-9]{1,2}.jpg>')
 @app.route('/cam_<regex("[0-9]+"):uid>.jpg')
+# ================ Swann / DVR16-1500
+@app.route('/Streaming/channels/<regex("[0-9]+"):uid>01/picture')
+
 @auth.login_required
 def video(uid=1):
     remote_ip=request.environ.get('HTTP_X_REAL_IP', request.remote_addr)
@@ -1282,4 +1468,8 @@ def gen(camera, remote_ip, blend=False, bigtext=True):
 if __name__ == '__main__':
 
     print("i... APP RUN FROM WEB.PY")
+    #
+    #
+    #
+    #
     app.run(host='0.0.0.0', port=config.CONFIG['netport'], threaded=True)
